from typing import Dict

from vents.notifiers.spec import NotificationSpec
from vents.notifiers.webhook import WebHookNotifier
from vents.providers.kinds import ProviderKind
from vents.settings import VENTS_CONFIG


class HipChatWebHookNotifier(WebHookNotifier):
    notification_key = ProviderKind.HIPCHAT
    name = "HipChat WebHook"
    description = "HipChat webhooks to send payload to a hipchat room."
    raise_empty_context = True

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        payload = {
            "message": notification.details,
            "message_format": "text",
            "color": notification.color,
            "from": VENTS_CONFIG.project_name,
        }

        return payload

    @classmethod
    def _prepare(cls, context: Dict) -> Dict:
        context = super()._prepare(context)

        return {
            "message": context.get("message"),
            "message_format": context.get("message_format", "html"),
            "color": context.get("color"),
            "from": context.get("from", VENTS_CONFIG.project_name),
            "attach_to": context.get("attach_to"),
            "notify": context.get("notify", False),
            "card": context.get("card"),
        }



import os

from typing import TYPE_CHECKING, Optional

from vents.providers.azure.base import (
    get_account_key,
    get_account_name,
    get_client_id,
    get_client_secret,
    get_connection_string,
    get_sas_token,
    get_tenant_id,
)
from vents.providers.base import BaseService

if TYPE_CHECKING:
    from vents.connections.connection import Connection


class AzureService(BaseService):
    account_name: Optional[str] = None
    account_key: Optional[str] = None
    connection_string: Optional[str] = None
    sas_token: Optional[str] = None
    tenant_id: Optional[str] = None
    client_id: Optional[str] = None
    client_secret: Optional[str] = None

    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"] = None
    ) -> Optional["AzureService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        schema = None
        builtin_env = None
        if connection:
            if connection.secret and connection.secret.mount_path:
                context_paths.append(connection.secret.mount_path)
            if connection.config_map and connection.config_map.mount_path:
                context_paths.append(connection.config_map.mount_path)
            if connection.schema_:
                schema = connection.get_schema_as_dict()
            if connection.env:
                builtin_env = connection.env

        account_name = get_account_name(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        account_key = get_account_key(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        connection_string = get_connection_string(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        sas_token = get_sas_token(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        tenant_id = get_tenant_id(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        client_id = get_client_id(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        client_secret = get_client_secret(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        return cls(
            account_name=account_name,
            account_key=account_key,
            connection_string=connection_string,
            sas_token=sas_token,
            tenant_id=tenant_id,
            client_id=client_id,
            client_secre=client_secret,
        )

    def set_env_vars(self):
        if self._account_name:
            os.environ["AZURE_ACCOUNT_NAME"] = self._account_name
        if self._account_key:
            os.environ["AZURE_ACCOUNT_KEY"] = self._account_key
        if self._connection_string:
            os.environ["AZURE_CONNECTION_STRING"] = self._connection_string



from vents.connections.catalog import ConnectionCatalog
from vents.connections.connection import Connection
from vents.connections.connection_resource import ConnectionResource
from vents.connections.connection_schema import (
    BucketConnection,
    ClaimConnection,
    GitConnection,
    HostConnection,
    HostPathConnection,
)



from typing import Dict

from vents.notifiers.spec import NotificationSpec
from vents.notifiers.webhook import WebHookNotifier
from vents.providers.kinds import ProviderKind
from vents.settings import VENTS_CONFIG


class SlackWebHookNotifier(WebHookNotifier):
    notification_key = ProviderKind.SLACK
    name = "Slack WebHook"
    description = "Slack webhooks to send payload to Slack Incoming Webhooks."
    raise_empty_context = True
    validate_keys = ["channel", "icon_url"]

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        fields = []  # Use build_field

        payload = {
            "fallback": notification.fallback,
            "title": notification.title,
            "title_link": notification.url,
            "text": notification.details,
            "fields": fields,
            "mrkdwn_in": ["text"],
            "footer_icon": VENTS_CONFIG.project_icon,
            "footer": VENTS_CONFIG.project_name,
            "color": notification.color,
            "ts": notification.ts,
        }

        return payload

    @classmethod
    def _prepare(cls, context):
        context = super()._prepare(context)

        data = {
            "fallback": context.get("fallback"),
            "title": context.get("title"),
            "title_link": context.get("title_link"),
            "text": context.get("text"),
            "fields": context.get("fields"),
            "mrkdwn_in": context.get("mrkdwn_in"),
            "footer_icon": context.get("footer_icon"),
            "footer": context.get("footer", VENTS_CONFIG.project_name),
            "color": context.get("color"),
        }
        return {"attachments": [data]}

    @classmethod
    def _pre_execute_web_hook(cls, data, config):
        channel = config.get("channel")
        icon_url = config.get("channel")
        if channel:
            data["channel"] = channel

        if icon_url:
            data["icon_url"] = icon_url

        return data



import os

from typing import TYPE_CHECKING, Optional

from vents.providers.base import BaseHttpService, BaseService
from vents.settings import VENTS_CONFIG

if TYPE_CHECKING:
    from vents.connections.connection import Connection


class SlackService(BaseService):
    token: Optional[str] = None
    is_async: bool = False

    @classmethod
    def load_from_connection(
        cls,
        connection: Optional["Connection"] = None,
    ) -> Optional["SlackService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        schema = None
        builtin_env = None
        if connection:
            if connection.secret and connection.secret.mount_path:
                context_paths.append(connection.secret.mount_path)
            if connection.config_map and connection.config_map.mount_path:
                context_paths.append(connection.config_map.mount_path)
            if connection.schema_:
                schema = connection.get_schema_as_dict()
            if connection.env:
                builtin_env = connection.env

        token = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["SLACK_TOKEN"],
        )
        return cls(token=token)

    def _set_session(self):
        from slack import AsyncWebClient, WebClient

        self._session = (
            AsyncWebClient(token=self.token)
            if self.is_async
            else WebClient(token=self.token)
        )

    def set_env_vars(self):
        if self.token:
            os.environ["SLACK_TOKEN"] = self.token


class SlackWebhookService(BaseService):
    url: Optional[str] = None
    is_async: bool = False

    @classmethod
    def load_from_connection(
        cls,
        connection: Optional["Connection"] = None,
    ) -> Optional["SlackWebhookService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        if connection:
            if connection.secret and connection.secret.url:
                context_paths.append(connection.secret.url)
        url = VENTS_CONFIG.read_keys(context_paths=context_paths, keys=["SLACK_URL"])
        return cls(url=url)

    def _set_session(self):
        from slack import AsyncWebhookClient, WebhookClient

        self._session = (
            AsyncWebhookClient(url=self.url)
            if self.is_async
            else WebhookClient(url=self.url)
        )

    def set_env_vars(self):
        if self.url:
            os.environ["SLACK_URL"] = self.url


class SlackHttpWebhookService(BaseHttpService):
    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"] = None
    ) -> Optional["SlackHttpWebhookService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        if connection:
            if connection.secret and connection.secret.url:
                context_paths.append(connection.secret.url)
        url = VENTS_CONFIG.read_keys(context_paths=context_paths, keys=["SLACK_URL"])
        method = VENTS_CONFIG.read_keys(
            context_paths=context_paths, keys=["SLACK_METHOD"]
        )
        session_attrs = VENTS_CONFIG.read_keys(
            context_paths=context_paths, keys=["SLACK_SESSION_ATTRS"]
        )
        return cls(url=url, method=method, session_attrs=session_attrs)



import json
import os

from typing import TYPE_CHECKING, Any, List, Optional

from google.oauth2.service_account import Credentials

from vents.providers.base import BaseService
from vents.providers.gcp.base import (
    get_default_key_path,
    get_gc_credentials,
    get_key_path,
    get_keyfile_dict,
    get_project_id,
    get_scopes,
)

if TYPE_CHECKING:
    from google.cloud.storage.client import Client

    from vents.connections.connection import Connection


class GCPService(BaseService):
    project_id: Optional[str] = None
    key_path: Optional[str] = None
    keyfile_dict: Optional[str] = None
    scopes: Optional[List[str]] = None
    credentials: Optional[Credentials] = None
    client_info: Optional[Any] = None
    client_options: Optional[Any] = None
    encoding: Optional[str] = "utf-8"

    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"] = None
    ) -> Optional["GCPService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        schema = None
        builtin_env = None
        if connection:
            if connection.secret and connection.secret.mount_path:
                context_paths.append(connection.secret.mount_path)
            if connection.config_map and connection.config_map.mount_path:
                context_paths.append(connection.config_map.mount_path)
            if connection.schema_:
                schema = connection.get_schema_as_dict()
            if connection.env:
                builtin_env = connection.env

        project_id = get_project_id(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        key_path = get_key_path(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        keyfile_dict = get_keyfile_dict(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        scopes = get_scopes(context_paths=context_paths, schema=schema, env=builtin_env)
        credentials = get_gc_credentials(
            key_path=key_path,
            keyfile_dict=keyfile_dict,
            scopes=scopes,
        )
        return cls(
            project_id=project_id,
            key_path=key_path,
            keyfile_dict=keyfile_dict,
            scopes=scopes,
            credentials=credentials,
        )

    def _set_session(self):
        self._session = Client(
            project=self.project_id,
            credentials=self.credentials,
            client_info=self.client_info,
            client_options=self.client_options,
        )

    def set_env_vars(self):
        if self.key_path:
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = self.key_path
        elif self.keyfile_dict:
            key_path = get_default_key_path()
            with open(key_path, "w") as outfile:
                json.dump(self.keyfile_dict, outfile)
            os.environ["GOOGLE_APPLICATION_CREDENTIALS"] = key_path



import os

from typing import TYPE_CHECKING, Optional

from vents.providers.base import BaseHttpService, BaseService
from vents.settings import VENTS_CONFIG

if TYPE_CHECKING:
    from vents.connections.connection import Connection


class RedditService(BaseService):
    client_id: Optional[str] = None
    client_secret: Optional[str] = None
    user_agent: Optional[str] = None
    username: Optional[str] = None
    password: Optional[str] = None

    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"] = None
    ) -> Optional["RedditService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        schema = None
        builtin_env = None
        if connection:
            if connection.secret and connection.secret.mount_path:
                context_paths.append(connection.secret.mount_path)
            if connection.config_map and connection.config_map.mount_path:
                context_paths.append(connection.config_map.mount_path)
            if connection.schema_:
                schema = connection.get_schema_as_dict()
            if connection.env:
                builtin_env = connection.env

        client_id = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["REDDIT_CLIENT_ID"],
        )
        client_secret = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["REDDIT_CLIENT_SECRET"],
        )
        user_agent = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["REDDIT_USER_AGENT"],
        )
        username = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["REDDIT_USERNAME"],
        )
        password = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["REDDIT_PASSWORD"],
        )
        return cls(
            client_id=client_id,
            client_secret=client_secret,
            user_agent=user_agent,
            username=username,
            password=password,
        )

    def _set_session(self):
        import praw

        self._session = praw.Reddit(
            client_id=self.client_id,
            client_secret=self.client_secret,
            user_agent=self.user_agent,
            username=self.username,
            password=self.password,
        )

    def set_env_vars(self):
        if self.client_id:
            os.environ["REDDIT_CLIENT_ID"] = self.client_id
        if self.client_secret:
            os.environ["REDDIT_CLIENT_SECRET"] = self.client_secret
        if self.user_agent:
            os.environ["REDDIT_USER_AGENT"] = self.user_agent
        if self.username:
            os.environ["REDDIT_USERNAME"] = self.username
        if self.password:
            os.environ["REDDIT_PASSWORD"] = self.password


class RedditRssService(BaseHttpService):
    url: Optional[str] = "https://www.reddit.com/r/{subreddit}.rss"

    @classmethod
    def load_from_connection(
        cls,
        connection: Optional["Connection"] = None,
    ) -> Optional["RedditRssService"]:
        context_paths = []
        if connection:
            if connection.secret and connection.secret.url:
                context_paths.append(connection.secret.url)
        kwargs = {}
        url = VENTS_CONFIG.read_keys(
            context_paths=context_paths, keys=["REDDIT_RSS_URL"]
        )
        if url:
            kwargs["url"] = url
        return cls(**kwargs)



from typing import Dict

from vents.notifiers.spec import NotificationSpec
from vents.notifiers.webhook import WebHookNotifier
from vents.providers.kinds import ProviderKind
from vents.settings import VENTS_CONFIG


class DiscordWebHookNotifier(WebHookNotifier):
    notification_key = ProviderKind.DISCORD
    name = "Discord WebHook"
    description = "Discord webhooks to send payload to a discord room."
    raise_empty_context = True

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        return {"content": notification.details}

    @classmethod
    def _prepare(cls, context: Dict) -> Dict:
        context = super()._prepare(context)

        payload = {
            "username": VENTS_CONFIG.project_name,
            "avatar_url": context.get("avatar_url") or VENTS_CONFIG.project_icon,
            "tts": context.get("tts", False),
        }
        content = context.get("content")
        if content and len(content) <= 2000:
            payload["content"] = content
        else:
            raise VENTS_CONFIG.exception(
                "Discord content must non null and 2000 or fewer characters."
            )

        proxy = context.get("proxy")
        if proxy:
            payload["https"] = proxy
        return payload



import abc

from typing import TYPE_CHECKING, Any, Dict, Optional

from clipped.compact.pydantic import PrivateAttr
from clipped.config.schema import BaseSchemaModel
from clipped.utils.requests import create_session, safe_request

if TYPE_CHECKING:
    from vents.connections.catalog import ConnectionCatalog
    from vents.connections.connection import Connection


class BaseService(BaseSchemaModel):
    _session: Optional[Any] = PrivateAttr(default=None)

    class Config:
        extra = "allow"

    @classmethod
    def load_from_catalog(
        cls, connection_name: str, catalog: Optional["ConnectionCatalog"]
    ) -> Optional["BaseService"]:
        """Loads a new service from the catalog."""
        connection = cls.get_connection(
            connection_name=connection_name, catalog=catalog
        )
        return cls.load_from_connection(connection=connection)

    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"] = None
    ) -> Optional["BaseService"]:
        raise NotImplementedError

    @staticmethod
    def get_connection(
        connection_name: str, catalog: Optional["ConnectionCatalog"]
    ) -> Optional["Connection"]:
        if not catalog or not connection_name:
            return None
        return catalog.connections_by_names.get(connection_name)

    @property
    def session(self):
        if self._session is None:
            # Create session with defaults
            self._set_session()
        return self._session

    @abc.abstractmethod
    def _set_session(self):
        raise NotImplementedError


class BaseHttpService(BaseService):
    url: Optional[str] = None
    session_attrs: Optional[Dict] = None
    method: Optional[str] = None

    def _set_session(self):
        self._session = create_session(session_attrs=self.session_attrs)

    def execute(self, **kwargs):
        url = kwargs.pop("url", self.url)
        return safe_request(url=url, method=self.method, session=self.session, **kwargs)



import os

from typing import Any, Dict, List, Optional, Union

from clipped.utils.lists import to_list
from clipped.utils.urls import validate_url

from vents.notifiers.spec import NotificationSpec
from vents.settings import VENTS_CONFIG

ConfigType = Union[Dict, List[Dict]]


class BaseNotifier:
    notification_key = None
    name = None
    description = ""
    raise_empty_context = True
    check_config = True
    validate_keys = None

    @classmethod
    def _validate_config(cls, config: ConfigType) -> ConfigType:
        """Validate that a given config is valid for the current config."""
        if not config:
            return []
        args = cls.validate_keys or []
        return cls._get_valid_config(config, *args)

    @classmethod
    def _get_valid_config(cls, config, *fields) -> ConfigType:
        config = to_list(config)
        web_hooks = []
        for web_hook in config:
            if not web_hook.get("url"):
                VENTS_CONFIG.logger.warning(
                    "Settings contains a non compatible web hook: `%s`", web_hook
                )
                continue

            url = web_hook["url"]
            if not validate_url(url):
                raise VENTS_CONFIG.exception(
                    "{} received invalid URL `{}`.".format(cls.name, url)
                )

            method = web_hook.get("method", "POST")
            if not isinstance(method, str):
                raise VENTS_CONFIG.exception(
                    "{} received invalid method `{}`.".format(cls.name, method)
                )

            _method = method.upper()
            if _method not in ["GET", "POST"]:
                raise VENTS_CONFIG.exception(
                    "{} received non compatible method `{}`.".format(cls.name, method)
                )

            result_web_hook = {"url": url, "method": _method}
            for field in fields:
                if field in web_hook:
                    result_web_hook[field] = web_hook[field]
            web_hooks.append(result_web_hook)

        return web_hooks

    @classmethod
    def _get_config(
        cls,
    ) -> ConfigType:  # TODO: Move to service to get from catalog/env var
        """Getting config to execute an action.

        Currently we get config from env vars.

        should be a list of urls and potentially a method.

        If no method is given, then by default we use POST.
        """
        value = os.environ.get(cls.notification_key)
        if not value:
            raise VENTS_CONFIG.exception(
                "Could not validate config for notifier {}".format(cls.name)
            )

        return VENTS_CONFIG.config_parser.parse(Dict)(
            key=cls.notification_key, value=value, is_list=True
        )

    @classmethod
    def get_config(cls, config: ConfigType = None) -> ConfigType:
        config = config or cls._get_config()
        config = cls._validate_config(config)
        return config

    @classmethod
    def _prepare(cls, context: Dict) -> Dict:
        """This is where you should alter the context to fit the action.

        Default behavior will leave the context as it is.
        """
        if not context and cls.raise_empty_context:
            raise VENTS_CONFIG.exception(
                "{} received invalid payload context.".format(cls.name)
            )

        return context

    @classmethod
    def _execute(cls, data: Dict, config: Optional[ConfigType]) -> None:
        raise NotImplementedError

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        """Implementation for turning a notification info into actionable notification."""

    @classmethod
    def execute(
        cls,
        notification: NotificationSpec,
        config: Optional[ConfigType] = None,
    ) -> Any:
        if not notification:
            raise VENTS_CONFIG.exception(
                "Received an invalid run `{}`.".format(notification)
            )
        context = cls.serialize_notification_to_context(notification=notification)
        config = cls.get_config(config=config)
        if cls.check_config and not config:
            return False

        data = cls._prepare(context)
        try:
            result = cls._execute(data=data, config=config)
        except Exception as e:
            VENTS_CONFIG.logger.error(
                "Exception during the execution of the notifier %s. Error: %s",
                cls.name,
                e,
                exc_info=True,
            )
            return
        return result



from typing import Optional

from s3fs import S3FileSystem as BaseS3FileSystem

from vents.providers.aws.service import AWSService


class S3FileSystem(BaseS3FileSystem):
    retries = 5

    async def _ls(self, path: str, detail: bool = False, force: bool = False):
        return await super()._ls(path, detail=detail, refresh=force)


class S3Service(AWSService):
    def _set_session(
        self,
        asynchronous: Optional[bool] = False,
        use_listings_cache: Optional[bool] = False,
        **kwargs,
    ):
        config_kwargs = kwargs.get("config_kwargs", {})
        if self.region and "region_name" not in config_kwargs:
            config_kwargs["region_name"] = self.region
        client_kwargs = kwargs.get("client_kwargs", {})
        if self.verify_ssl is not None and "verify" not in client_kwargs:
            client_kwargs["verify"] = self.verify_ssl
        self._session = S3FileSystem(
            key=self.access_key_id,
            secret=self.secret_access_key,
            token=self.session_token,
            use_ssl=self.use_ssl,
            endpoint_url=self.endpoint_url,
            config_kwargs=config_kwargs,
            client_kwargs=client_kwargs,
            asynchronous=asynchronous,
            use_listings_cache=use_listings_cache,
            **kwargs,
        )

    def get_fs(
        self,
        asynchronous: Optional[bool] = False,
        use_listings_cache: Optional[bool] = False,
        **kwargs,
    ):
        self._set_session(
            asynchronous=asynchronous,
            use_listings_cache=use_listings_cache,
            **kwargs,
        )
        return self.session



from typing import Dict, List, Optional, Union

from clipped.compact.pydantic import Field, StrictStr
from clipped.config.schema import BaseSchemaModel
from clipped.types.ref_or_obj import RefField


class BucketConnection(BaseSchemaModel):
    _IDENTIFIER = "bucket"

    # TODO: Remove once the kind is not set in the compiler, because the schema is converted to a `dict`
    kind: Optional[StrictStr] = None
    bucket: StrictStr

    def patch(self, schema: "BucketConnection"):
        self.bucket = schema.bucket or self.bucket


class ClaimConnection(BaseSchemaModel):
    _IDENTIFIER = "volume_claim"

    # TODO: Remove once the kind is not set in the compiler, because the schema is converted to a `dict`
    kind: Optional[StrictStr] = None
    volume_claim: StrictStr = Field(alias="volumeClaim")
    mount_path: StrictStr = Field(alias="mountPath")
    read_only: Optional[bool] = Field(alias="readOnly", default=None)

    def patch(self, schema: "ClaimConnection"):  # type: ignore
        self.volume_claim = schema.volume_claim or self.volume_claim
        self.mount_path = schema.mount_path or self.mount_path
        self.read_only = schema.read_only or self.read_only


class HostPathConnection(BaseSchemaModel):
    _IDENTIFIER = "host_path"

    # TODO: Remove once the kind is not set in the compiler, because the schema is converted to a `dict`
    kind: Optional[StrictStr] = None
    host_path: StrictStr = Field(alias="hostPath")
    mount_path: StrictStr = Field(alias="mountPath")
    read_only: Optional[bool] = Field(alias="readOnly", default=None)

    def patch(self, schema: "HostPathConnection"):  # type: ignore
        self.host_path = schema.host_path or self.host_path
        self.mount_path = schema.mount_path or self.mount_path
        self.read_only = schema.read_only or self.read_only


class HostConnection(BaseSchemaModel):
    _IDENTIFIER = "host"

    # TODO: Remove once the kind is not set in the compiler, because the schema is converted to a `dict`
    kind: Optional[StrictStr] = None
    url: StrictStr
    insecure: Optional[bool] = None

    def patch(self, schema: "HostConnection"):  # type: ignore
        self.url = schema.url or self.url
        self.insecure = schema.insecure or self.insecure


class GitConnection(BaseSchemaModel):
    _IDENTIFIER = "git"

    # TODO: Remove once the kind is not set in the compiler, because the schema is converted to a `dict`
    kind: Optional[StrictStr] = None
    url: Optional[StrictStr] = None
    revision: Optional[StrictStr] = None
    flags: Optional[List[StrictStr]] = None

    def get_name(self):
        if self.url:
            return self.url.split("/")[-1].split(".")[0]
        return None

    def patch(self, schema: "GitConnection"):
        self.url = schema.url or self.url
        self.revision = schema.revision or self.revision
        self.flags = schema.flags or self.flags


def patch_git(schema: Union[Dict, BaseSchemaModel], git_schema: GitConnection):
    if isinstance(schema, BaseSchemaModel) and not isinstance(schema, GitConnection):
        schema = GitConnection.from_dict(schema.to_dict())
    if git_schema.url:
        setattr(schema, "url", git_schema.url)
    if git_schema.revision:
        setattr(schema, "revision", git_schema.revision)
    if git_schema.flags:
        setattr(schema, "flags", git_schema.flags)

    return schema


ConnectionSchema = Union[
    BucketConnection,
    ClaimConnection,
    HostPathConnection,
    HostConnection,
    GitConnection,
    Dict,
    RefField,
]



from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Literal

from clipped.compact.pydantic import Field, StrictStr
from clipped.config.schema import BaseSchemaModel
from clipped.types.ref_or_obj import RefField

from vents.connections.connection_resource import ConnectionResource
from vents.connections.connection_schema import ConnectionSchema
from vents.providers.kinds import ProviderKind


class Connection(BaseSchemaModel):
    _IDENTIFIER = "connection"

    name: StrictStr
    kind: ProviderKind
    is_alias: Optional[bool] = Field(alias="isAlias", exclude=True, default=None)
    description: Optional[StrictStr] = None
    tags: Optional[Union[List[StrictStr], RefField]] = None
    schema_: Optional[ConnectionSchema] = Field(alias="schema", default=None)
    secret: Optional[Union[ConnectionResource, RefField]] = None
    config_map: Optional[Union[ConnectionResource, RefField]] = Field(
        alias="configMap", default=None
    )
    env: Optional[
        Union[Dict[str, Any], List[Union[Tuple[str, str], List[str]]], RefField]
    ] = None
    annotations: Optional[Union[Dict, RefField]] = None

    def get_schema_as_dict(self) -> Dict[str, Any]:
        schema = self.schema_
        if hasattr(schema, "to_dict"):
            schema = schema.to_dict()

        return schema

    @classmethod
    def from_model(cls, model) -> "Connection":
        schema = model.schema_
        secret = model.secret
        config_map = model.config_map
        if hasattr(schema, "to_dict"):
            schema = schema.to_dict()
        if hasattr(secret, "to_dict"):
            secret = secret.to_dict()
        if hasattr(config_map, "to_dict"):
            config_map = config_map.to_dict()
        return Connection.from_dict(
            {
                "name": model.name,
                "kind": model.kind,
                "schema": schema,
                "secret": secret,
                "configMap": config_map,
                "env": model.env,
                "annotations": model.annotations,
            }
        )

    @property
    def store_path(self) -> str:
        if self.is_mount:
            return self.schema_.mount_path.rstrip("/")
        if self.is_bucket:
            bucket = self.schema_.bucket.rstrip("/")
            if self.is_wasb:
                from clipped import types

                from vents.settings import VENTS_CONFIG

                return VENTS_CONFIG.config_parser.parse(types.WASB)(
                    key="schema", value=bucket
                ).get_container_path()
            return bucket

    @property
    def is_mount(self) -> bool:
        return ProviderKind.is_mount(self.kind)

    @property
    def is_artifact(self) -> bool:
        return ProviderKind.is_artifact(self.kind)

    @property
    def is_host_path(self) -> bool:
        return ProviderKind.is_host_path(self.kind)

    @property
    def is_volume_claim(self) -> bool:
        return ProviderKind.is_volume_claim(self.kind)

    @property
    def is_bucket(self) -> bool:
        return ProviderKind.is_bucket(self.kind)

    @property
    def is_gcs(self) -> bool:
        return self.kind == ProviderKind.GCS

    @property
    def is_s3(self) -> bool:
        return self.kind == ProviderKind.S3

    @property
    def is_wasb(self) -> bool:
        return ProviderKind.is_wasb(self.kind)

    @staticmethod
    def get_requested_resources(
        resources: List[ConnectionResource],
        connections: List["Connection"],
        resource_key: Literal["secret", "config_map"],
    ) -> List[ConnectionResource]:
        resources = resources or []

        connections = connections or []
        # Create a set of all resources:
        #   * resources request by non managed connections
        #   * resources requested directly by the user
        requested_resources = [r for r in resources if r.is_requested]
        resource_ids = {s.name for s in requested_resources}
        for connection in connections:
            resource = getattr(connection, resource_key)
            if resource and resource.name not in resource_ids:
                resource_ids.add(resource.name)
                requested_resources.append(resource)

        return requested_resources



NAME = "vents"
VERSION = "0.6.2"
DESC = "Open source connections, integrations, alerting, and notification library."
URL = "https://github.com/mmourafiq/vents"
AUTHOR = "Mourad Mourafiq"
EMAIL = "mourad.mourafiq@gmail.com"
LICENSE = "Apache 2.0"



import logging
import os

from typing import Any, Dict, List, Optional, Set, Type, Union

from clipped.compact.pydantic import Field
from clipped.config.parser import ConfigParser
from clipped.config.schema import BaseSchemaModel
from clipped.types import Uri
from clipped.utils.paths import check_dirname_exists

from vents.connections import ConnectionCatalog
from vents.connections.connection import Connection
from vents.exceptions import VentError

_logger = logging.getLogger("vents.config.reader")


class AppConfig(BaseSchemaModel):
    project_name: Optional[str] = "Vents"
    project_url: Optional[Uri] = ""
    project_icon: Optional[Uri] = ""
    env_prefix: Optional[str] = "VENTS"
    context_path: Optional[str] = ""
    logger: Any = _logger
    exception: Type[Exception] = VentError
    config_parser: Type[ConfigParser] = Field(
        default=ConfigParser, alias="config_parser"
    )
    catalog: Optional[ConnectionCatalog] = None

    def __init__(self, **data: Any):
        super().__init__(**data)
        self.catalog = self.catalog or self.load_connections_catalog()

    def read_keys_from_env(
        self, keys: Union[Set[str], List[str], str], env: Optional[Dict] = None
    ) -> Optional[Any]:
        """
        Returns a variable from one of the list of keys based on the os.env.
        Args:
            keys: list(str). list of keys to check in the environment

        Returns:
            str | None
        """
        env = env or os.environ
        keys = keys or []
        if not isinstance(keys, (list, tuple, set)):
            keys = [keys]
        for key in keys:
            value = env.get(key)
            if value:
                if value.lower() == "true":
                    return True
                if value.lower() == "false":
                    return False
                return value
            # Prepend prefix if set
            if self.env_prefix:
                key = "{}_{}".format(self.env_prefix, key)
                value = env.get(key)
                if value:
                    return value

        return None

    def read_keys_from_path(
        self, context_paths: List[str], keys: Union[Set[str], List[str], str]
    ) -> Optional[Any]:
        """
        Returns a variable from one of the list of keys based on a base path.
        Args:
            context_paths: List[str], base path where to look for keys.
            keys: list(str). list of keys to check in the environment

        Returns:
            str | None
        """
        context_paths = context_paths or []
        context_paths = [
            c
            for c in context_paths
            if check_dirname_exists(c, is_dir=True, reraise=False)
        ]
        if not context_paths:
            return None

        keys = keys or []
        if not isinstance(keys, (list, tuple, set)):
            keys = [keys]  # type: ignore
        for key in keys:
            for context_path in context_paths:
                key_path = os.path.join(context_path, key)
                if not os.path.exists(key_path):
                    continue
                with open(key_path) as f:
                    value = f.read()
                    if value:
                        if value.lower() == "true":
                            return True
                        if value.lower() == "false":
                            return False
                        return value

        return None

    def read_keys_from_schema(
        self, schema: Dict, keys: Union[Set[str], List[str], str]
    ) -> Optional[Any]:
        """Reads keys from a schema

        Args:
            schema: Dict. schema to read keys from
            keys: list(str). list of keys to check in the environment

        Returns:
            str | None
        """
        if not schema:
            return None
        keys = keys or []
        if not isinstance(keys, (list, tuple, set)):
            keys = [keys]  # type: ignore
        for key in keys:
            value = schema.get(key)
            if value:
                if value.lower() == "true":
                    return True
                if value.lower() == "false":
                    return False
                return value

    def get_connections_catalog_env_name(self) -> str:
        env_name = "CONNECTIONS_CATALOG"
        if self.env_prefix:
            env_name = "{}_{}".format(self.env_prefix, env_name)
        return env_name

    @staticmethod
    def get_connections_catalog(
        connections: Optional[List[Connection]],
    ) -> Optional[ConnectionCatalog]:
        if not connections:
            return None
        return ConnectionCatalog(connections=connections)

    def set_connections_catalog(self, connections: Optional[List[Connection]]):
        self.catalog = self.get_connections_catalog(connections)

    def load_connections_catalog(self) -> Optional[ConnectionCatalog]:
        catalog_env_name = self.get_connections_catalog_env_name()
        connections_catalog = os.environ.get(catalog_env_name)
        if not connections_catalog:
            return None
        return ConnectionCatalog.read(connections_catalog, config_type=".json")

    def get_connection_for(self, name: Optional[str]) -> Optional[Connection]:
        """Checks if a connection has a mount path exported"""
        if not name or not self.catalog:
            return None

        return self.catalog.connections_by_names.get(name)

    def read_keys(
        self,
        keys: List[str],
        schema: Optional[Dict] = None,
        env: Optional[Dict] = None,
        context_paths: Optional[List[str]] = None,
    ) -> Optional[Any]:
        """Returns a variable by checking first a context path and then in the environment."""
        keys = (
            {k.lower() for k in keys}
            | {k.upper() for k in keys}
            | {"".join(k.lower().split("_")) for k in keys}
        )
        if context_paths:
            value = self.read_keys_from_path(context_paths=context_paths, keys=keys)
            if value is not None:
                return value
        if schema:
            value = self.read_keys_from_schema(schema=schema, keys=keys)
            if value is not None:
                return value
        if env:
            value = self.read_keys_from_env(keys=keys, env=env)
            if value is not None:
                return value
        return self.read_keys_from_env(keys=keys)



from typing import Dict

from vents.notifiers.spec import NotificationSpec
from vents.notifiers.webhook import WebHookNotifier
from vents.providers.kinds import ProviderKind
from vents.settings import VENTS_CONFIG


class MattermostWebHookNotifier(WebHookNotifier):
    notification_key = ProviderKind.MATTERMOST
    name = "Mattermost WebHook"
    description = "Mattermost webhooks to send payload to a Mattermost channel."
    raise_empty_context = True
    validate_keys = ["channel"]

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        payload = {
            "title": notification.title,
            "text": notification.details,
            "color": notification.color,
            "fields": [],
            "author_name": VENTS_CONFIG.project_name,
            "author_link": notification.url,
            "author_icon": VENTS_CONFIG.project_icon,
        }

        return payload

    @classmethod
    def _prepare(cls, context):
        context = super()._prepare(context)

        data = {
            "pretext": context.get("pretext"),
            "title": context.get("title"),
            "text": context.get("text"),
            "color": context.get("color"),
            "fields": context.get("fields"),
            "author_name": context.get("author_name", VENTS_CONFIG.project_name),
            "author_link": context.get("author_link", VENTS_CONFIG.project_url),
            "author_icon": context.get("author_icon", VENTS_CONFIG.project_icon),
        }
        return {"attachments": [data]}

    @classmethod
    def _pre_execute_web_hook(cls, data, config):
        channel = config.get("channel")
        if channel:
            data["channel"] = channel

        return data



import logging
import os

from typing import List, Optional, Union

from vents.settings import VENTS_CONFIG

logging.getLogger("azure").setLevel(logging.WARNING)
logging.getLogger("azure.storage").setLevel(logging.WARNING)
logging.getLogger("azure.storage.blob").setLevel(logging.WARNING)


def get_account_name(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AZURE_ACCOUNT_NAME"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_account_key(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AZURE_ACCOUNT_KEY"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_connection_string(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AZURE_CONNECTION_STRING"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_sas_token(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AZURE_SAS_TOKEN", "AZURE_STORAGE_SAS_TOKEN"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_tenant_id(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AZURE_TENANT_ID"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_client_id(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AZURE_CLIENT_ID"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_client_secret(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AZURE_CLIENT_SECRET"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def set_env_vars(
    account_name: Optional[str] = None,
    account_key: Optional[str] = None,
    connection_string: Optional[str] = None,
):
    if account_name:
        os.environ["AZURE_ACCOUNT_NAME"] = account_name
    if account_key:
        os.environ["AZURE_ACCOUNT_KEY"] = account_key
    if connection_string:
        os.environ["AZURE_CONNECTION_STRING"] = connection_string



import os

from collections.abc import Mapping
from typing import Dict, List, Optional, Union

import google.auth
import google.oauth2.service_account

from clipped.utils.json import orjson_loads
from google.oauth2.service_account import Credentials

from vents.settings import VENTS_CONFIG

DEFAULT_SCOPES = ["https://www.googleapis.com/auth/cloud-platform"]


def get_default_key_path():
    return "{}/.gc/gc-secret.json".format(VENTS_CONFIG.context_path or "/tmp")


def get_project_id(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
    **kwargs,
) -> Optional[str]:
    value = kwargs.get("project_id")
    if value:
        return value
    keys = keys or [
        "GC_PROJECT",
        "GOOGLE_PROJECT",
        "GC_PROJECT_ID",
        "GOOGLE_PROJECT_ID",
    ]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_key_path(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["GC_KEY_PATH", "GOOGLE_KEY_PATH", "GOOGLE_APPLICATION_CREDENTIALS"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_keyfile_dict(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[Dict]:
    keys = keys or ["GC_KEYFILE_DICT", "GOOGLE_KEYFILE_DICT"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_scopes(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[List[str]]:
    keys = keys or ["GC_SCOPES", "GOOGLE_SCOPES"]
    scopes = VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )
    scopes = VENTS_CONFIG.config_parser.parse(str)(
        key="scopes",
        value=scopes,
        is_optional=True,
        is_list=True,
    )
    return scopes or DEFAULT_SCOPES


def get_gc_credentials(
    key_path: Optional[str],
    keyfile_dict: Optional[Union[str, Dict]],
    scopes: Optional[List[str]],
) -> Credentials:
    """
    Returns the Credentials object for Google API
    """
    if not key_path and not keyfile_dict:
        context_secret = get_default_key_path()
        # Look for default GC path
        if os.path.exists(context_secret):
            key_path = context_secret

    if not key_path and not keyfile_dict:
        VENTS_CONFIG.logger.info(
            "Getting connection using `google.auth.default()` "
            "since no key file is defined for hook."
        )
        credentials, _ = google.auth.default(scopes=scopes)
    elif key_path:
        # Get credentials from a JSON file.
        if key_path.endswith(".json"):
            VENTS_CONFIG.logger.info("Getting connection using a JSON key file.")
            credentials = Credentials.from_service_account_file(
                os.path.abspath(key_path), scopes=scopes
            )
        else:
            raise VENTS_CONFIG.exception("Unrecognised extension for key file.")
    else:
        # Get credentials from JSON data.
        try:
            if not isinstance(keyfile_dict, Mapping):
                keyfile_dict = orjson_loads(keyfile_dict)  # type: ignore

            # Convert escaped newlines to actual newlines if any.
            keyfile_dict["private_key"] = keyfile_dict["private_key"].replace(
                "\\n", "\n"
            )

            credentials = Credentials.from_service_account_info(
                keyfile_dict, scopes=scopes
            )
        except ValueError:  # json.decoder.JSONDecodeError does not exist on py2
            raise VENTS_CONFIG.exception("Invalid key JSON.")

    return credentials



from typing import Optional

from vents.config import AppConfig

VENTS_CONFIG: Optional[AppConfig]


def create_app(config: Optional[AppConfig] = None) -> AppConfig:
    global VENTS_CONFIG

    VENTS_CONFIG = config or AppConfig()
    return VENTS_CONFIG



import os

from typing import TYPE_CHECKING, Optional

from vents.providers.aws.base import (
    get_aws_access_key_id,
    get_aws_assume_role,
    get_aws_role_arn,
    get_aws_secret_access_key,
    get_aws_security_token,
    get_aws_session_duration,
    get_aws_session_name,
    get_aws_use_ssl,
    get_aws_verify_ssl,
    get_endpoint_url,
    get_region,
)
from vents.providers.base import BaseService

if TYPE_CHECKING:
    from vents.connections.connection import Connection


class AWSService(BaseService):
    resource: Optional[str] = None
    region: Optional[str] = None
    endpoint_url: Optional[str] = None
    access_key_id: Optional[str] = None
    secret_access_key: Optional[str] = None
    session_token: Optional[str] = None
    verify_ssl: Optional[bool] = None
    use_ssl: Optional[bool] = None

    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"] = None
    ) -> Optional["AWSService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        schema = None
        builtin_env = None
        if connection:
            if connection.secret and connection.secret.mount_path:
                context_paths.append(connection.secret.mount_path)
            if connection.config_map and connection.config_map.mount_path:
                context_paths.append(connection.config_map.mount_path)
            if connection.schema_:
                schema = connection.get_schema_as_dict()
            if connection.env:
                builtin_env = connection.env

        region = get_region(context_paths=context_paths, schema=schema, env=builtin_env)
        endpoint_url = get_endpoint_url(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        sts_endpoint_url = get_endpoint_url(
            keys=["AWS_STS_ENDPOINT_URL"],
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
        )
        access_key_id = get_aws_access_key_id(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        secret_access_key = get_aws_secret_access_key(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        verify_ssl = get_aws_verify_ssl(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        use_ssl = get_aws_use_ssl(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        session_token = get_aws_security_token(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        assume_role = get_aws_assume_role(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        role_arn = get_aws_role_arn(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        session_name = get_aws_session_name(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        session_duration = get_aws_session_duration(
            context_paths=context_paths, schema=schema, env=builtin_env
        )
        if assume_role and role_arn:
            credentials = cls.assume_role(
                role_arn=role_arn,
                session_name=session_name,
                session_duration=session_duration,
                region=region,
                endpoint_url=sts_endpoint_url,
                access_key_id=access_key_id,
                secret_access_key=secret_access_key,
                verify_ssl=verify_ssl,
                use_ssl=use_ssl,
            )
            access_key_id = credentials["AccessKeyId"]
            secret_access_key = credentials["SecretAccessKey"]
            session_token = credentials["SessionToken"]
        return cls(
            region=region,
            endpoint_url=endpoint_url,
            access_key_id=access_key_id,
            secret_access_key=secret_access_key,
            session_token=session_token,
            verify_ssl=verify_ssl,
            use_ssl=use_ssl,
        )

    @classmethod
    def assume_role(
        cls,
        role_arn: str,
        session_name: Optional[str] = None,
        session_duration: Optional[int] = 43200,
        region: Optional[str] = None,
        endpoint_url: Optional[str] = None,
        access_key_id: Optional[str] = None,
        secret_access_key: Optional[str] = None,
        verify_ssl: Optional[bool] = None,
        use_ssl: Optional[bool] = None,
    ) -> "AWSService":
        import botocore.session

        session = botocore.session.get_session()
        endpoint_url = endpoint_url or "https://sts.{}.amazonaws.com".format(region)
        client = session.create_client(
            "sts",
            region_name=region,
            use_ssl=use_ssl,
            verify=verify_ssl,
            aws_access_key_id=access_key_id,
            aws_secret_access_key=secret_access_key,
            endpoint_url=endpoint_url,
        )
        response = client.assume_role(
            RoleArn=role_arn,
            RoleSessionName=session_name or "S3Session",
            DurationSeconds=session_duration or 43200,
        )
        return response["Credentials"]

    def _set_session(self):
        import boto3

        self._session = boto3.session.Session(
            aws_access_key_id=self.access_key_id,
            aws_secret_access_key=self.secret_access_key,
            aws_session_token=self.session_token,
            region_name=self.region,
        )

    def set_env_vars(self):
        if self.endpoint_url:
            os.environ["AWS_ENDPOINT_URL"] = self.endpoint_url
        if self.access_key_id:
            os.environ["AWS_ACCESS_KEY_ID"] = self.access_key_id
        if self.secret_access_key:
            os.environ["AWS_SECRET_ACCESS_KEY"] = self.secret_access_key
        if self.session_token:
            os.environ["AWS_SECURITY_TOKEN"] = self.session_token
        if self.region:
            os.environ["AWS_REGION"] = self.region
        if self.use_ssl is not None:
            os.environ["AWS_USE_SSL"] = str(self.use_ssl)
        if self.verify_ssl is not None:
            os.environ["AWS_VERIFY_SSL"] = str(self.verify_ssl)

    def get_client(self):
        return self.session.client(
            service_name=self.resource,
            region_name=self.region,
            endpoint_url=self.endpoint_url,
            use_ssl=self.use_ssl,
            verify=self.verify_ssl,
        )

    def get_resource(self):
        return self.session.resource(
            self.resource,
            endpoint_url=self.endpoint_url,
            use_ssl=self.use_ssl,
            verify=self.verify_ssl,
        )



import os

from typing import TYPE_CHECKING, Dict, Optional

from clipped.utils.json import orjson_dumps, orjson_loads

from vents.providers.base import BaseService
from vents.settings import VENTS_CONFIG

if TYPE_CHECKING:
    from vents.connections.connection import Connection


class OpenAIService(BaseService):
    api_key: Optional[str] = None
    base_url: Optional[str] = None
    is_async: bool = False
    kwargs: Optional[Dict] = None

    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"] = None
    ) -> Optional["OpenAIService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        schema = None
        builtin_env = None
        if connection:
            if connection.secret and connection.secret.mount_path:
                context_paths.append(connection.secret.mount_path)
            if connection.config_map and connection.config_map.mount_path:
                context_paths.append(connection.config_map.mount_path)
            if connection.schema_:
                schema = connection.get_schema_as_dict()
            if connection.env:
                builtin_env = connection.env

        api_key = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["OPENAI_API_KEY"],
        )
        base_url = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["OPENAI_BASE_URL"],
        )
        kwargs = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["OPENAI_KWARGS"],
        )
        if kwargs and not isinstance(kwargs, dict):
            kwargs = orjson_loads(kwargs)
        return cls(
            api_key=api_key,
            base_url=base_url,
            kwargs=kwargs,
        )

    def _set_session(self):
        import openai

        kwargs = self.kwargs or {}
        self._session = (
            openai.AsyncOpenAI(
                api_key=self.api_key,
                base_url=self.base_url,
                **kwargs,
            )
            if self.is_async
            else openai.OpenAI(
                api_key=self.api_key,
                base_url=self.base_url,
                **kwargs,
            )
        )

    def set_env_vars(self):
        if self.api_key:
            os.environ["OPENAI_API_KEY"] = self.api_key
        if self.base_url:
            os.environ["OPENAI_BASE_URL"] = self.base_url
        if self.kwargs:
            os.environ["OPENAI_KWARGS"] = orjson_dumps(self.kwargs)



from typing import Dict, List, Optional

from clipped.compact.pydantic import PrivateAttr
from clipped.config.schema import BaseSchemaModel

from vents.connections.connection import Connection
from vents.connections.connection_resource import ConnectionResource


class ConnectionCatalog(BaseSchemaModel):
    connections: Optional[List[Connection]] = None
    _all_connections: List[Connection] = PrivateAttr()
    _secrets: Optional[List[ConnectionResource]] = PrivateAttr(default=None)
    _config_maps: Optional[List[ConnectionResource]] = PrivateAttr(default=None)
    _connections_by_names: Dict[str, Connection] = PrivateAttr()

    def __init__(
        self,
        **data,
    ):
        super().__init__(**data)
        # Post init
        self._all_connections = []
        self._connections_by_names = {}
        self._secrets = None
        self._config_maps = None
        self.set_all_connections()

    def set_all_connections(self) -> None:
        self._all_connections = self.connections[:] if self.connections else []
        self._connections_by_names = {}
        self._secrets = None
        self._config_maps = None

    @property
    def all_connections(self) -> List[Connection]:
        return self._all_connections

    @property
    def secrets(self) -> List[ConnectionResource]:
        if self._secrets or not self._all_connections:
            return self._secrets
        secret_names = set()
        secrets = []
        for c in self._all_connections:
            if c.secret and c.secret.name not in secret_names:
                secret_names.add(c.secret.name)
                secrets.append(c.secret)
        self._secrets = secrets
        return self._secrets

    @property
    def config_maps(self) -> List[ConnectionResource]:
        if self._config_maps or not self._all_connections:
            return self._config_maps
        config_map_names = set()
        config_maps = []
        for c in self._all_connections:
            if c.config_map and c.config_map.name not in config_map_names:
                config_map_names.add(c.config_map.name)
                config_maps.append(c.config_map)
        self._config_maps = config_maps
        return self._config_maps

    @property
    def connections_by_names(self) -> Dict[str, Connection]:
        if self._connections_by_names or not self._all_connections:
            return self._connections_by_names

        self._connections_by_names = {c.name: c for c in self._all_connections}
        return self._connections_by_names



from datetime import datetime
from typing import Dict, Optional

from clipped.config.schema import BaseSchemaModel
from clipped.types import Uri


class NotificationSpec(BaseSchemaModel):
    title: str
    description: str
    details: str
    fallback: Optional[str] = None
    context: Optional[Dict] = None
    url: Optional[Uri] = None
    color: Optional[str] = None
    ts: Optional[datetime] = None



from vents.notifiers.discord_webhook import DiscordWebHookNotifier
from vents.notifiers.hipchat_webhook import HipChatWebHookNotifier
from vents.notifiers.mattermost_webhook import MattermostWebHookNotifier
from vents.notifiers.pagerduty_webhook import PagerDutyWebHookNotifier
from vents.notifiers.slack_webhook import SlackWebHookNotifier
from vents.notifiers.spec import NotificationSpec
from vents.notifiers.webhook import WebHookNotifier

NOTIFIERS = {
    DiscordWebHookNotifier.notification_key: DiscordWebHookNotifier,
    HipChatWebHookNotifier.notification_key: HipChatWebHookNotifier,
    MattermostWebHookNotifier.notification_key: MattermostWebHookNotifier,
    PagerDutyWebHookNotifier.notification_key: PagerDutyWebHookNotifier,
    SlackWebHookNotifier.notification_key: SlackWebHookNotifier,
    WebHookNotifier.notification_key: WebHookNotifier,
}



from typing import List, Optional, Union

from clipped.utils.bools import to_bool

from vents.settings import VENTS_CONFIG


def get_aws_access_key_id(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AWS_ACCESS_KEY_ID"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_aws_secret_access_key(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AWS_SECRET_ACCESS_KEY"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_aws_security_token(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AWS_SECURITY_TOKEN"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_region(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AWS_REGION"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_endpoint_url(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AWS_ENDPOINT_URL"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_aws_use_ssl(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[bool]:
    keys = keys or ["AWS_USE_SSL"]
    value = VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore
    if value is not None:
        return to_bool(value)
    return True


def get_aws_verify_ssl(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[bool]:
    keys = keys or ["AWS_VERIFY_SSL"]
    value = VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore
    if value is not None:
        return to_bool(value)
    return None


def get_aws_session(
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
):
    import boto3

    aws_access_key_id = get_aws_access_key_id(
        context_paths=context_paths, schema=schema, env=env
    )
    aws_secret_access_key = get_aws_secret_access_key(
        context_paths=context_paths, schema=schema, env=env
    )
    aws_session_token = get_aws_security_token(
        context_paths=context_paths, schema=schema, env=env
    )
    region_name = get_region(context_paths=context_paths, schema=schema, env=env)
    return boto3.session.Session(
        aws_access_key_id=aws_access_key_id,
        aws_secret_access_key=aws_secret_access_key,
        aws_session_token=aws_session_token,
        region_name=region_name,
    )


def get_aws_client(
    client_type,
    context_paths: Optional[List[str]] = None,
):
    session = get_aws_session(
        context_paths=context_paths,
    )
    endpoint_url = get_endpoint_url(context_paths=context_paths)
    aws_use_ssl = get_aws_use_ssl(context_paths=context_paths)
    aws_verify_ssl = get_aws_verify_ssl(context_paths=context_paths)
    return session.client(
        client_type,
        endpoint_url=endpoint_url,
        use_ssl=aws_use_ssl,
        verify=aws_verify_ssl,
    )


def get_aws_resource(
    resource_type,
    schema: Optional[str] = None,
    env: Optional[str] = None,
    context_paths: Optional[List[str]] = None,
):
    session = get_aws_session(context_paths=context_paths, schema=schema, env=env)
    endpoint_url = get_endpoint_url(context_paths=context_paths)
    aws_use_ssl = get_aws_use_ssl(context_paths=context_paths)
    aws_verify_ssl = get_aws_verify_ssl(context_paths=context_paths)
    return session.resource(
        resource_type,
        endpoint_url=endpoint_url,
        use_ssl=aws_use_ssl,
        verify=aws_verify_ssl,
    )


def get_aws_assume_role(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[bool]:
    keys = keys or ["AWS_ASSUME_ROLE"]
    value = VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore
    if value is not None:
        return to_bool(value)
    return None


def get_aws_role_arn(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AWS_ROLE_ARN"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_aws_session_name(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AWS_SESSION_NAME"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore


def get_aws_session_duration(
    keys: Optional[Union[str, List[str]]] = None,
    context_paths: Optional[List[str]] = None,
    schema: Optional[str] = None,
    env: Optional[str] = None,
) -> Optional[str]:
    keys = keys or ["AWS_SESSION_DURATION"]
    return VENTS_CONFIG.read_keys(
        context_paths=context_paths, schema=schema, env=env, keys=keys
    )  # type: ignore



from typing import Dict

from vents.notifiers.spec import NotificationSpec
from vents.notifiers.webhook import WebHookNotifier
from vents.providers.kinds import ProviderKind
from vents.settings import VENTS_CONFIG


class PagerDutyWebHookNotifier(WebHookNotifier):
    notification_key = ProviderKind.PAGERDUTY
    name = "PagerDuty WebHook"
    description = "PagerDuty webhooks to send event payload to pagerduty."
    raise_empty_context = True
    validate_keys = ["service_key"]

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        payload = {
            "event_type": notification.title,
            "description": notification.description,
            "details": notification.details,
            "incident_key": "trigger",
            "client": VENTS_CONFIG.project_name,
            "client_url": notification.url,
            "contexts": [],
        }

        return payload

    @classmethod
    def _prepare(cls, context: Dict) -> Dict:
        context = super()._prepare(context)

        return {
            "event_type": context.get("event_type"),
            "description": context.get("description"),
            "details": context.get("details"),
            "incident_key": context.get("incident_key"),
            "client": context.get("client", VENTS_CONFIG.project_name),
            "client_url": context.get("client_url", VENTS_CONFIG.project_url),
            "contexts": context.get("contexts"),
        }

    @classmethod
    def _pre_execute_web_hook(cls, data: Dict, config: Dict) -> Dict:
        service_key = config.get("service_key")
        if service_key:
            data["service_key"] = service_key

        return data



from typing import Optional

from gcsfs import GCSFileSystem as BaseGCSFileSystem

from vents.providers.gcp.service import GCPService


class GCSFileSystem(BaseGCSFileSystem):
    retries = 5

    async def set_session(self):
        return await self._set_session()


class GCSService(GCPService):
    def _set_session(
        self,
        asynchronous: Optional[bool] = False,
        use_listings_cache: Optional[bool] = False,
        **kwargs,
    ):
        self._session = GCSFileSystem(
            project=self.project_id,
            token=self.credentials,
            asynchronous=asynchronous,
            use_listings_cache=use_listings_cache,
            **kwargs,
        )

    def get_fs(
        self,
        asynchronous: Optional[bool] = False,
        use_listings_cache: Optional[bool] = False,
        **kwargs,
    ):
        self._set_session(
            asynchronous=asynchronous, use_listings_cache=use_listings_cache, **kwargs
        )
        return self.session



import os

from typing import TYPE_CHECKING, Optional

from vents.providers.base import BaseService
from vents.settings import VENTS_CONFIG

if TYPE_CHECKING:
    from vents.connections.connection import Connection


class GithubService(BaseService):
    token: Optional[str] = None
    host: Optional[str] = None

    @classmethod
    def load_from_connection(
        cls, connection: Optional["Connection"] = None
    ) -> Optional["GithubService"]:
        # Check if there are mounting based on secrets/configmaps
        context_paths = []
        schema = None
        builtin_env = None
        if connection:
            if connection.secret and connection.secret.token:
                context_paths.append(connection.secret.token)
            if connection.config_map and connection.config_map.host:
                context_paths.append(connection.config_map.host)
            if connection.schema_:
                schema = connection.get_schema_as_dict()
            if connection.env:
                builtin_env = connection.env

        token = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["GITHUB_TOKEN"],
        )
        host = VENTS_CONFIG.read_keys(
            context_paths=context_paths,
            schema=schema,
            env=builtin_env,
            keys=["GITHUB_HOST"],
        )
        return cls(token=token, host=host)

    def _set_session(self):
        kwargs = {"base_url": self.host} if self.host else {}
        from github import Github

        self._session = Github(login_or_token=self.token, **kwargs)

    def set_env_vars(self):
        if self.token:
            os.environ["GITHUB_TOKEN"] = self.token



from typing import List, Optional

from clipped.compact.pydantic import Field, StrictStr
from clipped.config.schema import BaseSchemaModel


class ConnectionResource(BaseSchemaModel):
    _IDENTIFIER = "connection_resource"

    name: StrictStr
    mount_path: Optional[StrictStr] = Field(alias="mountPath", default=None)
    host_path: Optional[StrictStr] = Field(alias="hostPath", default=None)
    items: Optional[List[StrictStr]] = None
    default_mode: Optional[str] = Field(alias="defaultMode", default=None)
    is_requested: Optional[bool] = Field(alias="isRequested", default=None)



from requests import RequestException
from typing import Dict, List

from clipped.utils.requests import safe_request

from vents.notifiers.base import BaseNotifier
from vents.notifiers.spec import NotificationSpec
from vents.providers.kinds import ProviderKind
from vents.settings import VENTS_CONFIG


class WebHookNotifier(BaseNotifier):
    notification_key = ProviderKind.WEBHOOK
    name = "WebHook"
    description = (
        "Webhooks send an HTTP payload to the webhook's configured URL."
        "Webhooks can be used automatically "
        "by subscribing to certain events, "
        "or manually triggered by a user operation."
    )
    raise_empty_context = False

    @classmethod
    def serialize_notification_to_context(cls, notification: NotificationSpec) -> Dict:
        context = {
            "context": notification.context,
            "title": notification.title,
            "details": notification.details,
            "finished_at": notification.ts,
        }
        return context

    @classmethod
    def _pre_execute_web_hook(cls, data: Dict, config: Dict) -> Dict:
        return data

    @classmethod
    def _execute(cls, data: Dict, config: List[Dict]) -> None:
        for web_hook in config:
            data = cls._pre_execute_web_hook(data=data, config=web_hook)
            try:
                if web_hook["method"] == "POST":
                    safe_request(
                        url=web_hook["url"], method=web_hook["method"], json=data
                    )
                else:
                    safe_request(
                        url=web_hook["url"], method=web_hook["method"], params=data
                    )
            except RequestException:
                VENTS_CONFIG.logger.warning(
                    "Could not send web hook, exception.", exc_info=True
                )



from clipped.utils.enums import PEnum


class ProviderKind(str, PEnum):
    HOST_PATH = "host_path"
    VOLUME_CLAIM = "volume_claim"
    GCS = "gcs"
    S3 = "s3"
    WASB = "wasb"
    REGISTRY = "registry"
    GIT = "git"
    AWS = "aws"
    GCP = "gcp"
    AZURE = "azure"
    MYSQL = "mysql"
    POSTGRES = "postgres"
    ORACLE = "oracle"
    VERTICA = "vertica"
    SQLITE = "sqlite"
    MSSQL = "mssql"
    REDIS = "redis"
    PRESTO = "presto"
    MONGO = "mongo"
    CASSANDRA = "cassandra"
    FTP = "ftp"
    GRPC = "grpc"
    HDFS = "hdfs"
    HTTP = "http"
    PIG_CLI = "pig_cli"
    HIVE_CLI = "hive_cli"
    HIVE_METASTORE = "hive_metastore"
    HIVE_SERVER2 = "hive_server2"
    JDBC = "jdbc"
    JENKINS = "jenkins"
    SAMBA = "samba"
    SNOWFLAKE = "snowflake"
    SSH = "ssh"
    CLOUDANT = "cloudant"
    DATABRICKS = "databricks"
    SEGMENT = "segment"
    SLACK = "slack"
    DISCORD = "discord"
    MATTERMOST = "mattermost"
    PAGERDUTY = "pagerduty"
    HIPCHAT = "hipchat"
    WEBHOOK = "webhook"
    CUSTOM = "custom"

    @classmethod
    def mount_values(cls):
        return {cls.HOST_PATH, cls.VOLUME_CLAIM}

    @classmethod
    def blob_values(cls):
        return {cls.GCS, cls.S3, cls.WASB}

    @classmethod
    def artifact_values(cls):
        return cls.blob_values() | cls.mount_values()

    @classmethod
    def host_values(cls):
        return {cls.GIT, cls.REGISTRY}

    @classmethod
    def is_bucket(cls, kind):
        return kind in cls.blob_values()

    @classmethod
    def is_mount(cls, kind):
        return kind in cls.mount_values()

    @classmethod
    def is_host_path(cls, kind):
        return kind == cls.HOST_PATH

    @classmethod
    def is_volume_claim(cls, kind):
        return kind == cls.VOLUME_CLAIM

    @classmethod
    def is_artifact(cls, kind):
        return kind in cls.artifact_values()

    @classmethod
    def is_git(cls, kind):
        return kind == cls.GIT

    @classmethod
    def is_ssh(cls, kind):
        return kind == cls.SSH

    @classmethod
    def is_registry(cls, kind):
        return kind == cls.REGISTRY

    @classmethod
    def is_s3(cls, kind):
        return kind == cls.S3

    @classmethod
    def is_wasb(cls, kind):
        return kind == cls.WASB

    @classmethod
    def is_gcs(cls, kind):
        return kind == cls.GCS



from typing import Optional

from adlfs import AzureBlobFileSystem as BaseAzureBlobFileSystem

from vents.providers.azure.service import AzureService


class AzureBlobFileSystem(BaseAzureBlobFileSystem):
    async def _put_file(self, lpath, rpath, delimiter="/", overwrite=True, **kwargws):
        return await super()._put_file(
            lpath, rpath, delimiter=delimiter, overwrite=overwrite, **kwargws
        )

    async def _ls(
        self,
        path: str,
        force: bool = False,
        delimiter: str = "/",
        return_glob: bool = False,
        **kwargs,
    ):
        invalidate_cache = kwargs.pop("invalidate_cache", force)
        return await super()._ls(
            path,
            invalidate_cache=invalidate_cache,
            delimiter=delimiter,
            return_glob=return_glob,
            **kwargs,
        )


class BlobStorageService(AzureService):
    def _set_session(
        self,
        asynchronous: Optional[bool] = False,
        use_listings_cache: Optional[bool] = False,
        **kwargs,
    ):
        self._session = AzureBlobFileSystem(
            account_name=self.account_name,
            account_key=self.account_key,
            connection_string=self.connection_string,
            sas_token=self.sas_token,
            tenant_id=self.tenant_id,
            client_id=self.client_id,
            client_secret=self.client_secret,
            asynchronous=asynchronous,
            use_listings_cache=use_listings_cache,
            **kwargs,
        )

    def get_fs(
        self,
        asynchronous: Optional[bool] = False,
        use_listings_cache: Optional[bool] = False,
        **kwargs,
    ):
        self._set_session(
            asynchronous=asynchronous, use_listings_cache=use_listings_cache, **kwargs
        )
        return self.session
